@@ -1,21 +1,28 @@
 package Net::GitHub::V3::Repos;
 
-use Any::Moose;
+use Moo;
 
-our $VERSION = '0.53';
+our $VERSION = '0.64';
 our $AUTHORITY = 'cpan:FAYLAND';
 
+use Carp;
 use URI::Escape;
+use URI;
 use HTTP::Request::Common qw(POST);
 
 with 'Net::GitHub::V3::Query';
 
 sub list {
-    my ( $self, $type ) = @_;
-    $type ||= 'all';
-    my $u = '/user/repos';
-    $u .= '?type=' . $type if $type ne 'all';
-    return $self->query($u);
+    my ( $self, $args ) = @_;
+
+    # for old
+    unless (ref($args) eq 'HASH') {
+        $args = { type => $args };
+    }
+
+    my $uri = URI->new('/user/repos');
+    $uri->query_form($args);
+    return $self->query($uri->as_string);
 }
 
 sub list_all {
@@ -27,12 +34,17 @@ sub list_all {
 }
 
 sub list_user {
-    my ($self, $user, $type) = @_;
+    my ($self, $user, $args) = @_;
     $user ||= $self->u;
-    $type ||= 'all';
-    my $u = "/users/" . uri_escape($user) . "/repos";
-    $u .= '?type=' . $type if $type ne 'all';
-    return $self->query($u);
+
+    # for old
+    unless (ref($args) eq 'HASH') {
+        $args = { type => $args };
+    }
+
+    my $uri = URI->new("/users/" . uri_escape($user) . "/repos");
+    $uri->query_form($args);
+    return $self->query($uri->as_string);
 }
 
 sub list_org {
@@ -55,6 +67,63 @@ sub create {
     return $self->query('POST', $u, $data);
 }
 
+sub upload_asset {
+    my $self = shift;
+    unshift @_, $self->u, $self->repo if @_ < 5;
+    my ($user, $repos, $release_id, $name, $content_type, $file_content) = @_;
+
+    my $ua = $self->ua;
+    my $url = $self->upload_url . "/repos/$user/$repos/releases/$release_id/assets?name=" . uri_escape($name);
+    my $req = HTTP::Request->new( 'POST', $url );
+    $req->accept_decodable;
+    $req->content($file_content);
+    $req->header( 'Content-Type', $content_type );
+
+    my $res = $ua->request($req);
+
+    my $data;
+    if ($res->header('Content-Type') and $res->header('Content-Type') =~ 'application/json') {
+        my $json = $res->decoded_content;
+        $data = eval { $self->json->jsonToObj($json) };
+        unless ($data) {
+            # We tolerate bad JSON for errors,
+            # otherwise we just rethrow the JSON parsing problem.
+            die unless $res->is_error;
+            $data = { message => $res->message };
+        }
+    } else {
+        $data = { message => $res->message };
+    }
+
+    return wantarray ? %$data : $data;
+}
+
+sub commits {
+    my $self = shift;
+    if (@_ < 2) {
+        unshift @_, $self->repo;
+        unshift @_, $self->u;
+    }
+    my ($user, $repos, $args) = @_;
+
+    my $uri = URI->new("/repos/" . uri_escape($user) . "/" . uri_escape($repos) . '/commits');
+    $uri->query_form($args);
+    return $self->query($uri->as_string);
+}
+
+sub list_deployments {
+    my $self = shift;
+    if (@_ < 2) {
+        unshift @_, $self->repo;
+        unshift @_, $self->u;
+    }
+    my ($user, $repos, $args) = @_;
+
+    my $uri = URI->new("/repos/" . uri_escape($user) . "/" . uri_escape($repos) . '/deployments');
+    $uri->query_form($args);
+    return $self->query($uri->as_string);
+}
+
 ## build methods on fly
 my %__methods = (
 
@@ -65,6 +134,8 @@ my %__methods = (
     teams     => { url => "/repos/%s/%s/teams" },
     tags      => { url => "/repos/%s/%s/tags" },
     branches  => { url => "/repos/%s/%s/branches" },
+    branch => { url => "/repos/%s/%s/branches/%s" },
+    delete => { url => "/repos/%s/%s", method => 'DELETE', check_status => 204 },
 
     # http://developer.github.com/v3/repos/collaborators/
     collaborators       => { url => "/repos/%s/%s/collaborators" },
@@ -73,7 +144,6 @@ my %__methods = (
     delete_collaborator => { url => "/repos/%s/%s/collaborators/%s", method => 'DELETE', check_status => 204 },
 
     # http://developer.github.com/v3/repos/commits/
-    commits  => { url => "/repos/%s/%s/commits" },
     commit   => { url => "/repos/%s/%s/commits/%s" },
     comments => { url => "/repos/%s/%s/comments" },
     comment  => { url => "/repos/%s/%s/comments/%s" },
@@ -92,6 +162,18 @@ my %__methods = (
     download  => { url => "/repos/%s/%s/downloads/%s" },
     delete_download => { url => "/repos/%s/%s/downloads/%s", method => 'DELETE', check_status => 204 },
 
+    # http://developer.github.com/v3/repos/releases/
+    releases => { url => "/repos/%s/%s/releases" },
+    release  => { url => "/repos/%s/%s/releases/%s" },
+    create_release => { url => "/repos/%s/%s/releases", method => 'POST', args => 1 },
+    update_release => { url => "/repos/%s/%s/releases/%s", method => 'PATCH', args => 1 },
+    delete_release => { url => "/repos/%s/%s/releases/%s", method => 'DELETE', check_status => 204 },
+
+    release_assets => { url => "/repos/%s/%s/releases/%s/assets" },
+    release_asset => { url => "/repos/%s/%s/releases/%s/assets/%s" },
+    update_release_asset => { url => "/repos/%s/%s/releases/%s/assets/%s", method => 'PATCH', args => 1 },
+    delete_release_asset => { url => "/repos/%s/%s/releases/%s/assets/%s", method => 'DELETE', check_status => 204 },
+
     forks => { url => "/repos/%s/%s/forks" },
 
     # http://developer.github.com/v3/repos/keys/
@@ -107,6 +189,13 @@ my %__methods = (
     watch   => { url => "/user/watched/%s/%s", is_u_repo => 1, method => 'PUT', check_status => 204 },
     unwatch => { url => "/user/watched/%s/%s", is_u_repo => 1, method => 'DELETE', check_status => 204 },
 
+    subscribers   => { url => "/repos/%s/%s/subscribers" },
+    subscription  => { url => "/repos/%s/%s/subscription" },
+    is_subscribed => { url => "/repos/%s/%s/subscription", check_status => 200 },
+    subscribe     => { url => "/repos/%s/%s/subscription", method => 'PUT',
+        check_status => 200, args => 1 },
+    unsubscribe   => { url => "/repos/%s/%s/subscription", method => 'DELETE', check_status => 204 },
+
     # http://developer.github.com/v3/repos/hooks/
     hooks => { url => "/repos/%s/%s/hooks" },
     hook  => { url => "/repos/%s/%s/hooks/%s" },
@@ -121,6 +210,18 @@ my %__methods = (
     # http://developer.github.com/v3/repos/statuses/
     list_statuses => { url => "/repos/%s/%s/statuses/%s" },
     create_status => { url => "/repos/%s/%s/statuses/%s", method => 'POST', args => 1 },
+
+    # https://developer.github.com/v3/repos/deployments
+    create_deployment => { url => "/repos/%s/%s/deployments", method => 'POST', args => 1 },
+    create_deployment_status => { url => "/repos/%s/%s/deployments/%s/statuses", method => 'POST', args => 1 },
+    list_deployment_statuses => { url => "/repos/%s/%s/deployments/%s/statuses", method => 'GET'},
+
+    contributor_stats => { url => "/repos/%s/%s/stats/contributors", method => 'GET'},
+    commit_activity => { url => "/repos/%s/%s/stats/commit_activity", method => 'GET'},
+    code_frequency => { url => "/repos/%s/%s/stats/code_frequency", method => 'GET'},
+    participation => { url => "/repos/%s/%s/stats/participation", method => 'GET'},
+    punch_card => { url => "/repos/%s/%s/stats/punch_card", method => 'GET'},
+
 );
 __build_methods(__PACKAGE__, %__methods);
 
@@ -199,8 +300,7 @@ sub watched {
     return $self->query($u);
 }
 
-no Any::Moose;
-__PACKAGE__->meta->make_immutable;
+no Moo;
 
 1;
 __END__
@@ -245,9 +345,14 @@ L<http://developer.github.com/v3/repos/>
 =item list_org
 
     my @rp = $repos->list; # or my $rp = $repos->list;
-    my @rp = $repos->list('private');
+    my @rp = $repos->list({
+        type => 'private'
+        sort => 'updated'
+    });
     my @rp = $repos->list_user('c9s');
-    my @rp = $repos->list_user('c9s', 'member');
+    my @rp = $repos->list_user('c9s', {
+        type => 'member'
+    });
     my @rp = $repos->list_org('perlchina');
     my @rp = $repos->list_org('perlchina', 'public');
 
@@ -291,6 +396,10 @@ B<To ease the keyboard, we provied two ways to call any method which starts with
 
     $repos->update({ homepage => 'https://metacpan.org/module/Net::GitHub' });
 
+=item delete
+
+    $repos->delete();
+
 =item contributors
 
 =item languages
@@ -306,6 +415,7 @@ B<To ease the keyboard, we provied two ways to call any method which starts with
     my @teams = $repos->teams;
     my @tags = $repos->tags;
     my @branches = $repos->branches;
+    my $branch = $repos->branch('master');
 
 =back
 
@@ -341,6 +451,9 @@ L<http://developer.github.com/v3/repos/commits/>
 =item commit
 
     my @commits = $repos->commits;
+    my @commits = $repos->commits({
+        author => 'fayland'
+    });
     my $commit  = $repos->commit($sha);
 
 =item comments
@@ -376,45 +489,6 @@ L<http://developer.github.com/v3/repos/commits/>
 
 =back
 
-=head3 Downloads
-
-L<http://developer.github.com/v3/repos/downloads/>
-
-=over 4
-
-=item downloads
-
-=item download
-
-=item delete_download
-
-    my @downloads = $repos->downloads;
-    my $download  = $repos->download($download_id);
-    my $st = $repos->delete_download($download_id);
-
-=item create_download
-
-=item upload_download
-
-    my $download = $repos->create_download( {
-        "name" => "new_file.jpg",
-        "size" => 114034,
-        "description" => "Latest release",
-        "content_type" => "text/plain"
-    } );
-    my $st = $repos->upload_download($download, "/path/to/new_file.jpg");
-
-    # or batch it
-    my $st = $repos->create_download( {
-        "name" => "new_file.jpg",
-        "size" => 114034,
-        "description" => "Latest release",
-        "content_type" => "text/plain",
-        file => '/path/to/new_file.jpg',
-    } );
-
-=back
-
 =head3 Forks API
 
 L<http://developer.github.com/v3/repos/forks/>
@@ -492,6 +566,49 @@ L<http://developer.github.com/v3/repos/watching/>
 
 =back
 
+=head3 Subscriptions
+
+Github changed the ideas of Watchers (stars) and Subscriptions (new watchers).
+
+    https://github.com/blog/1204-notifications-stars
+
+The Watchers code in this module predates the terminology change, so the new
+Watcher methods use the GitHub 'subscription' terminology.
+
+=over 4
+
+=item subscribers
+
+Returns a list of subscriber data hashes.
+
+=item is_subscribed
+
+Returns true or false if you are subscribed
+
+    $repos->is_subscribed();
+    $repos->is_subscribed('fayland','perl-net-github');
+
+=item subscription
+
+Returns more information about your subscription to a repo.
+is_subscribed is a shortcut to calling this and checking for
+subscribed => 1.
+
+=item subscribe
+
+Required argument telling github if you want to subscribe or if you want
+to ignore mentions. If you want to change from subscribed to ignores you
+need to unsubscribe first.
+
+    $repos->subscribe('fayland','perl-net-github', { subscribed => 1 })
+    $repos->subscribe('fayland','perl-net-github', { ignored => 1 })
+
+=item unsubscribe
+
+    $repos->unsubscribe('fayland','perl-net-github');
+
+=back
+
 =head3 Hooks API
 
 L<http://developer.github.com/v3/repos/hooks/>
@@ -555,6 +672,127 @@ L<http://developer.github.com/v3/repos/statuses/>
 
 =back
 
+=head3 Repo Releases API
+
+L<http://developer.github.com/v3/repos/releases/>
+
+=over 4
+
+=item releases
+
+    my @releases = $repos->releases();
+
+=item release
+
+    my $release = $repos->release($release_id);
+
+=item create_release
+
+    my $release = $repos->create_release({
+      "tag_name" => "v1.0.0",
+      "target_commitish" => "master",
+      "name" => "v1.0.0",
+      "body" => "Description of the release",
+      "draft" => \1,
+    });
+
+=item update_release
+
+    my $release = $repos->update_release($release_id, {
+      "tag_name" => "v1.0.0",
+      "target_commitish" => "master",
+      "name" => "v1.0.0",
+      "body" => "Description of the release",
+    });
+
+=item delete_release
+
+    $repos->delete_release($release_id);
+
+=item release_assets
+
+    my @release_assets = $repos->release_assets($release_id);
+
+=item upload_asset
+
+    my $asset = $repos->upload_asset($release_id, $name, $content_type, $file_content);
+
+Check examples/upload_asset.pl for a working example.
+
+=item release_asset
+
+    my $release_asset = $repos->release_asset($release_id, $asset_id);
+
+=item update_release_asset
+
+    my $release_asset = $repos->update_release_asset($release_id, $asset_id, {
+        name" => "foo-1.0.0-osx.zip",
+        "label" => "Mac binary"
+    });
+
+=item delete_release_asset
+
+    my $ok = $repos->delete_release_asset($release_id, $asset_id);
+
+=back
+
+=head3 Repo Deployment API
+
+L<http://developer.github.com/v3/repos/deployments/>
+
+=over 4
+
+=item list_deployments
+
+    my $response = $repos->list_deployments( $owner, $repo, {
+        'ref' => 'feature-branch',
+    });
+
+=item create_deployment
+
+    my $response = $repos->create_deployment( $owner, $repo, {
+      "ref" => 'feature-branch',
+      "description" => "deploying my new feature",
+    });
+
+=item list_deployment_statuses
+
+    my $response = $repos->list_deployment_statuses( $owner, $repo, $deployment_id );
+
+=item create_deployment_status
+
+    my $response = $repos->create_deployment_status( $owner, $repo, $deployment_id, {
+        "state": "success",
+        "target_url": "https://example.com/deployment/42/output",
+        "description": "Deployment finished successfully."
+    });
+
+=back
+
+=head3 Repo Statistics API
+
+L<http://developer.github.com/v3/repos/statistics/>
+
+=over 4
+
+=item contributor stats
+
+=item commit activity
+
+=item code frequency
+
+=item participation
+
+=item punch card
+
+    my $contributor_stats   = $repos->contributor_stats($owner, $repo);
+    my $commit_activity     = $repos->commit_activity($owner, $repo);
+    my $code_freq           = $repos->code_frequency($owner, $repo);
+    my $participation       = $repos->participation($owner, $repo);
+    my $punch_card          = $repos->punch_card($owner, $repo);
+
+=back
+
 =head1 AUTHOR & COPYRIGHT & LICENSE
 
-Refer L<Net::GitHub>
\ No newline at end of file
+Refer L<Net::GitHub>
